#include <Adafruit_NeoPixel.h>
#include <OneWire.h>

#define LED_STRIP_PIN   6
#define NUM_LEDS        9
#define TEMPERATURE_PIN 11
#define FAN_PIN         3

Adafruit_NeoPixel strip = Adafruit_NeoPixel(NUM_LEDS, LED_STRIP_PIN, NEO_GRB + NEO_KHZ800);
OneWire  ds(TEMPERATURE_PIN);

void setup() {
  Serial.begin(9600);
  pinMode(TEMPERATURE_PIN, INPUT);
  pinMode(FAN_PIN, OUTPUT);
  pinMode(LED_STRIP_PIN, OUTPUT);
  analogReference(INTERNAL);
  strip.begin();
  strip.show();
}

void loop() {
  byte i;
  byte present = 0;
  byte data[12];
  byte addr[8];

 if ( !ds.search(addr)) {
    ds.reset_search();
    delay(250);
    return;
  }
  
  present = ds.reset();
  ds.select(addr);    
  ds.write(0x44);         // Read Scratchpad

  delay(1000);
  
  present = ds.reset();
  ds.select(addr);    
  ds.write(0xBE);         // Read Scratchpad

  for ( i = 0; i < 9; i++) {           // we need 9 bytes
    data[i] = ds.read();
  }
  uint16_t val = data[0] | ( (uint16_t) data[1] << 8);
  float temperature = val * 0.0625; // Convert ADC value to temperature in Celsius

  Serial.print("\nTemperature: ");
  Serial.print(temperature);

  if (temperature < 24) {
    analogWrite(FAN_PIN, 255);
    setColor(255, 255, 255); // White
  } else if (temperature >= 24 && temperature < 26) {
    analogWrite(FAN_PIN, 120); // 30% duty cycle
    setColor(255, 255, 0);    // Yellow
  } else if (temperature >= 26 && temperature < 28) {
    analogWrite(FAN_PIN, 60); // 60% duty cycle
    setColor(255, 165, 0);     // Orange
  } else {
    analogWrite(FAN_PIN, 0); // 100% duty cycle
    setColor(255, 0, 0);       // Red
  }

  delay(1000); // Read temperature and update fan speed every second
}

void setColor(uint8_t r, uint8_t g, uint8_t b) {
  for (int i = 0; i < NUM_LEDS; i++) {
    strip.setPixelColor(i, strip.Color(r, g, b));
  }
  strip.show();
}
